/*
 *  pv_display_lists_X11.c
 *  ProteinViewer
 *
 *  Created by Naoto Morikawa on 6/16/06.
 *  Copyright (C) 2006, 2007, 2008 Naoto Morikawa. All rights reserved.
 *
 *  2006-10-01 version 0.1 release
 *  2006-11-20 version 0.2 release
 *  2007-04-09 support CO, CB, CG, CU, and CV in create_display_list()
 *  2007-04-15 version 0.3 release
 *  2007-04-09 support CH, CX, and CK in create_display_list()
 *  2008-07-29 version 0.6 release 
 */

#include <stdlib.h>
#include <GL/glut.h>
#include <stdio.h>
#include <math.h>
#include "pv_main.h"
#include "pv_primitives.h"
#include "pv_display_lists.h"

extern struct pe_coor_system  my_coor;
extern float Mtile_coor[];

GLuint tetra_table[24];
GLuint tetra_table2[24];
GLuint tetra_tablew[24];
GLuint tetra_table2w[24];

/* tetrahedron table for rendering */
void set_tetra_table(void)
{
	tetra_table[XYZW] = tetra_xyzw;
	tetra_table[WXYZ] = tetra_xyzw;
	tetra_table[ZWXY] = tetra_xyzw;
	tetra_table[YZWX] = tetra_xyzw;
	tetra_table[XYWZ] = tetra_xywz;
	tetra_table[ZXYW] = tetra_xywz;
	tetra_table[WZXY] = tetra_xywz;
	tetra_table[YWZX] = tetra_xywz;
	tetra_table[XZYW] = tetra_xzyw;
	tetra_table[WXZY] = tetra_xzyw;
	tetra_table[YWXZ] = tetra_xzyw;
	tetra_table[ZYWX] = tetra_xzyw;
	tetra_table[XZWY] = tetra_xzwy;
	tetra_table[YXZW] = tetra_xzwy;
	tetra_table[WYXZ] = tetra_xzwy;
	tetra_table[ZWYX] = tetra_xzwy;
	tetra_table[XWYZ] = tetra_xwyz;
	tetra_table[ZXWY] = tetra_xwyz;
	tetra_table[YZXW] = tetra_xwyz;
	tetra_table[WYZX] = tetra_xwyz;
	tetra_table[XWZY] = tetra_xwzy;
	tetra_table[YXWZ] = tetra_xwzy;
	tetra_table[ZYXW] = tetra_xwzy;
	tetra_table[WZYX] = tetra_xwzy;
	
	tetra_table2[XYZW] = tetra2_xyzw;
	tetra_table2[WXYZ] = tetra2_xyzw;
	tetra_table2[ZWXY] = tetra2_xyzw;
	tetra_table2[YZWX] = tetra2_xyzw;
	tetra_table2[XYWZ] = tetra2_xywz;
	tetra_table2[ZXYW] = tetra2_xywz;
	tetra_table2[WZXY] = tetra2_xywz;
	tetra_table2[YWZX] = tetra2_xywz;
	tetra_table2[XZYW] = tetra2_xzyw;
	tetra_table2[WXZY] = tetra2_xzyw;
	tetra_table2[YWXZ] = tetra2_xzyw;
	tetra_table2[ZYWX] = tetra2_xzyw;
	tetra_table2[XZWY] = tetra2_xzwy;
	tetra_table2[YXZW] = tetra2_xzwy;
	tetra_table2[WYXZ] = tetra2_xzwy;
	tetra_table2[ZWYX] = tetra2_xzwy;
	tetra_table2[XWYZ] = tetra2_xwyz;
	tetra_table2[ZXWY] = tetra2_xwyz;
	tetra_table2[YZXW] = tetra2_xwyz;
	tetra_table2[WYZX] = tetra2_xwyz;
	tetra_table2[XWZY] = tetra2_xwzy;
	tetra_table2[YXWZ] = tetra2_xwzy;
	tetra_table2[ZYXW] = tetra2_xwzy;
	tetra_table2[WZYX] = tetra2_xwzy;
	/*****************************/
	tetra_tablew[XYZW] = tetraw_xyzw;
	tetra_tablew[WXYZ] = tetraw_xyzw;
	tetra_tablew[ZWXY] = tetraw_xyzw;
	tetra_tablew[YZWX] = tetraw_xyzw;
	tetra_tablew[XYWZ] = tetraw_xywz;
	tetra_tablew[ZXYW] = tetraw_xywz;
	tetra_tablew[WZXY] = tetraw_xywz;
	tetra_tablew[YWZX] = tetraw_xywz;
	tetra_tablew[XZYW] = tetraw_xzyw;
	tetra_tablew[WXZY] = tetraw_xzyw;
	tetra_tablew[YWXZ] = tetraw_xzyw;
	tetra_tablew[ZYWX] = tetraw_xzyw;
	tetra_tablew[XZWY] = tetraw_xzwy;
	tetra_tablew[YXZW] = tetraw_xzwy;
	tetra_tablew[WYXZ] = tetraw_xzwy;
	tetra_tablew[ZWYX] = tetraw_xzwy;
	tetra_tablew[XWYZ] = tetraw_xwyz;
	tetra_tablew[ZXWY] = tetraw_xwyz;
	tetra_tablew[YZXW] = tetraw_xwyz;
	tetra_tablew[WYZX] = tetraw_xwyz;
	tetra_tablew[XWZY] = tetraw_xwzy;
	tetra_tablew[YXWZ] = tetraw_xwzy;
	tetra_tablew[ZYXW] = tetraw_xwzy;
	tetra_tablew[WZYX] = tetraw_xwzy;
	
	tetra_table2w[XYZW] = tetra2w_xyzw;
	tetra_table2w[WXYZ] = tetra2w_xyzw;
	tetra_table2w[ZWXY] = tetra2w_xyzw;
	tetra_table2w[YZWX] = tetra2w_xyzw;
	tetra_table2w[XYWZ] = tetra2w_xywz;
	tetra_table2w[ZXYW] = tetra2w_xywz;
	tetra_table2w[WZXY] = tetra2w_xywz;
	tetra_table2w[YWZX] = tetra2w_xywz;
	tetra_table2w[XZYW] = tetra2w_xzyw;
	tetra_table2w[WXZY] = tetra2w_xzyw;
	tetra_table2w[YWXZ] = tetra2w_xzyw;
	tetra_table2w[ZYWX] = tetra2w_xzyw;
	tetra_table2w[XZWY] = tetra2w_xzwy;
	tetra_table2w[YXZW] = tetra2w_xzwy;
	tetra_table2w[WYXZ] = tetra2w_xzwy;
	tetra_table2w[ZWYX] = tetra2w_xzwy;
	tetra_table2w[XWYZ] = tetra2w_xwyz;
	tetra_table2w[ZXWY] = tetra2w_xwyz;
	tetra_table2w[YZXW] = tetra2w_xwyz;
	tetra_table2w[WYZX] = tetra2w_xwyz;
	tetra_table2w[XWZY] = tetra2w_xwzy;
	tetra_table2w[YXWZ] = tetra2w_xwzy;
	tetra_table2w[ZYXW] = tetra2w_xwzy;
	tetra_table2w[WZYX] = tetra2w_xwzy;
	return;
}

void create_display_list(void)
{
	int i;
	float dTheta, Theta;
	double tetra_facter, tetra_zfacter;
	double tetra_facter2, tetra_zfacter2;
	
	dTheta = 2 * PI / 20.0;
	Theta = dTheta;
	tetra_facter2 = sqrt(3.0) * my_coor.axes[0].length / sqrt(6.0);
	tetra_zfacter2 = sqrt(3.0) * my_coor.axes[0].length / (2.0 * sqrt(3.0));
	tetra_facter = 1.6 * my_coor.axes[0].length / sqrt(6.0);
	tetra_zfacter = 1.6 * my_coor.axes[0].length / (2 * sqrt(3.0));
	if (0) {
		fprintf(STDERR,
			"create_display_l ist(): tetra_factor = %8.3f\n",
			tetra_facter);
		fprintf(STDERR,
			"create_display_list(): tetra_zfactor = %8.3f\n",
			tetra_zfacter);
	}
	
	sphere_white = glGenLists(1);
	if (sphere_white == 0) goto err_handler;
	SPHERE_LIST(sphere_white, white, polymer_shininess, 0.15);
	sphere_50white = glGenLists(1);
	if (sphere_50white == 0) goto err_handler;
	SPHERE_LIST(sphere_50white, white, polymer_shininess, 0.50);
	sphere_blue = glGenLists(1);
	if (sphere_blue == 0) goto err_handler;
	SPHERE_LIST(sphere_blue, blue, polymer_shininess, 0.15);
	sphere_red = glGenLists(1);
	if (sphere_red == 0) goto err_handler;
	SPHERE_LIST(sphere_red, red, polymer_shininess, 0.15);
	sphere_green = glGenLists(1);
	if (sphere_green == 0) goto err_handler;
	SPHERE_LIST(sphere_green, green, polymer_shininess, 0.15);
	sphere_black = glGenLists(1);			// 2008-07-29
	if (sphere_black == 0) goto err_handler;
	SPHERE_LIST(sphere_black, black, polymer_shininess, 0.15);
	sphere_gray = glGenLists(1);
	if (sphere_gray == 0) goto err_handler;
	SPHERE_LIST(sphere_gray, gray, polymer_shininess, 0.15);
	sphere_30blue = glGenLists(1);
	if (sphere_30blue == 0) goto err_handler;
	SPHERE_LIST(sphere_30blue, blue, polymer_shininess, 0.30);
	sphere_30darkblue = glGenLists(1);
	if (sphere_30darkblue == 0) goto err_handler;
	SPHERE_LIST(sphere_30darkblue, darkblue, polymer_shininess, 0.30);
	sphere_30red = glGenLists(1);
	if (sphere_30red == 0) goto err_handler;
	SPHERE_LIST(sphere_30red, red, polymer_shininess, 0.30);
	sphere_yellow = glGenLists(1);
	if (sphere_yellow == 0) goto err_handler;
	SPHERE_LIST(sphere_yellow, yellow, polymer_shininess, 0.10);
	sphere_30yellow = glGenLists(1);
	if (sphere_30yellow == 0) goto err_handler;
	SPHERE_LIST(sphere_30yellow, yellow, polymer_shininess, 0.30);
	sphere_30orange = glGenLists(1);	/* NN 2007-04-09 */
	if (sphere_30orange == 0) goto err_handler;
	SPHERE_LIST(sphere_30orange, orange, polymer_shininess, 0.30);
	sphere_30white = glGenLists(1);
	if (sphere_30white == 0) goto err_handler;
	SPHERE_LIST(sphere_30white, white, polymer_shininess, 0.30);
	sphere_60red = glGenLists(1);
	if (sphere_60red == 0) goto err_handler;
	SPHERE_LIST(sphere_60red, red, polymer_shininess, 0.60);
	sphere_60yellow = glGenLists(1);
	if (sphere_60yellow == 0) goto err_handler;
	SPHERE_LIST(sphere_60yellow, yellow, polymer_shininess, 0.60);
	sphere_60orange = glGenLists(1);	/* NM 2007-04-09 */
	if (sphere_60orange == 0) goto err_handler;
	SPHERE_LIST(sphere_60orange, orange, polymer_shininess, 0.60);
	sphere_60blue = glGenLists(1);	/* NM 2007-04-09 */
	if (sphere_60blue == 0) goto err_handler;
	SPHERE_LIST(sphere_60blue, blue, polymer_shininess, 0.60);
	sphere_60green = glGenLists(1);	/* NM 2007-04-09 */
	if (sphere_60green == 0) goto err_handler;
	SPHERE_LIST(sphere_60green, green, polymer_shininess, 0.60);
	sphere_60white = glGenLists(1);
	if (sphere_60white == 0) goto err_handler;
	SPHERE_LIST(sphere_60white, white2, polymer_shininess, 0.60);
	
	sphere_60black = glGenLists(1);	/* NM 2008-07-29 */
	if (sphere_60black == 0) goto err_handler;
	SPHERE_LIST(sphere_60black, black2, polymer_shininess, 0.60);
	
	sphere_30tetra2 = glGenLists(1);
	if (sphere_30tetra2 == 0) goto err_handler;
	SPHERE_LIST(sphere_30tetra2, tetra_color2, polymer_shininess, 0.60);
	sphere_tetra1 = glGenLists(1);
	if (sphere_tetra1 == 0) goto err_handler;
	SPHERE_LIST(sphere_tetra1, tetra_color1, polymer_shininess, 0.30);
	
	/* CYLINDER */
	cylinder_2 = glGenLists(1);
	if (cylinder_2 == 0) goto err_handler;
	CYLINDER_LIST(cylinder_2, polymer_diffuse, polymer_specular,
		      dark_shininess, 0.02);
	cylinder_5 = glGenLists(1);
	if (cylinder_5 == 0) goto err_handler;
	CYLINDER_LIST(cylinder_5, polymer_diffuse, polymer_specular,
		      dark_shininess, 0.05);
	cylinder_10 = glGenLists(1);
	if (cylinder_10 == 0) goto err_handler;
	CYLINDER_LIST(cylinder_10, polymer_diffuse, polymer_specular,
		      dark_shininess, 0.10);
	cylinder_1010 = glGenLists(1);
	if (cylinder_1010 == 0) goto err_handler;
	CYLINDER_LIST(cylinder_1010, polymer_diffuse, polymer_specular,
		      polymer_shininess, 0.10);
	cylinder_15 = glGenLists(1);
	if (cylinder_15 == 0) goto err_handler;
	CYLINDER_LIST(cylinder_15, polymer_diffuse, polymer_specular,
		      dark_shininess, 0.15);
	cylinder_1015 = glGenLists(1);
	if (cylinder_1015 == 0) goto err_handler;
	CYLINDER_LIST(cylinder_1015, blue, blue, dark_shininess, 0.15);
	cylinder_2015 = glGenLists(1);
	if (cylinder_2015 == 0) goto err_handler;
	CYLINDER_LIST(cylinder_2015, tetra_color2, tetra_color2, dark_shininess, 0.15);
	cylinder_30 = glGenLists(1);
	if (cylinder_30 == 0) goto err_handler;
	CYLINDER_LIST(cylinder_30, blue, blue, dark_shininess, 0.30);
	
	
	/** SOLID TETRAHEDRONS in the cube of vertices
	 *  (1,1,1)/sqrt(3), (-1,1,1)/sqrt(3), (-1,-1,1)/sqrt(3),
	 *  (1,-1,1)/sqrt(3), (1,1,-1)/sqrt(3), (-1,1,-1)/sqrt(3),
	 *  (-1,-1,-1)/sqrt(3), and (1,-1,-1)/sqrt(3)
	 */
	tetra_xzyw = glGenLists(1);
	if (tetra_xzyw == 0) goto err_handler;
	TETRA_LIST(tetra_xzyw,   0.0, 0.0, 1.0, 0.0, tetra_color1,
		   polymer_shininess, glutSolidTetrahedron);
	tetra_xwyz = glGenLists(1);
	if (tetra_xwyz == 0) goto err_handler;
	TETRA_LIST(tetra_xwyz, 180.0, 0.0, 1.0, 0.0, tetra_color1,
		   polymer_shininess, glutSolidTetrahedron);
	tetra_xzwy = glGenLists(1);
	if (tetra_xzwy == 0) goto err_handler;
	TETRA_LIST(tetra_xzwy, -90.0, 1.0, 0.0, 0.0, tetra_color1,
		   polymer_shininess, glutSolidTetrahedron);
	tetra_xywz = glGenLists(1);
	if (tetra_xywz == 0) goto err_handler;
	TETRA_LIST(tetra_xywz,  90.0, 1.0, 0.0, 0.0, tetra_color1,
		   polymer_shininess, glutSolidTetrahedron);
	tetra_xwzy = glGenLists(1);
	if (tetra_xwzy == 0) goto err_handler;
	TETRA_LIST(tetra_xwzy, -90.0, 0.0, 1.0, 0.0, tetra_color1,
		   polymer_shininess, glutSolidTetrahedron);
	tetra_xyzw = glGenLists(1);
	if (tetra_xyzw == 0) goto err_handler;
	TETRA_LIST(tetra_xyzw,  90.0, 0.0, 1.0, 0.0, tetra_color1,
		   polymer_shininess, glutSolidTetrahedron);
	
	tetra2_xzyw = glGenLists(1);
	if (tetra2_xzyw == 0) goto err_handler;
	TETRA_LIST(tetra2_xzyw,   0.0, 0.0, 1.0, 0.0, tetra_color2,
		   polymer_shininess, glutSolidTetrahedron);
	tetra2_xwyz = glGenLists(1);
	if (tetra2_xwyz == 0) goto err_handler;
	TETRA_LIST(tetra2_xwyz, 180.0, 0.0, 1.0, 0.0, tetra_color2,
		   polymer_shininess, glutSolidTetrahedron);
	tetra2_xzwy = glGenLists(1);
	if (tetra2_xzwy == 0) goto err_handler;
	TETRA_LIST(tetra2_xzwy, -90.0, 1.0, 0.0, 0.0, tetra_color2,
		   polymer_shininess, glutSolidTetrahedron);
	tetra2_xywz = glGenLists(1);
	if (tetra2_xywz == 0) goto err_handler;
	TETRA_LIST(tetra2_xywz,  90.0, 1.0, 0.0, 0.0, tetra_color2,
		   polymer_shininess, glutSolidTetrahedron);
	tetra2_xwzy = glGenLists(1);
	if (tetra2_xwzy == 0) goto err_handler;
	TETRA_LIST(tetra2_xwzy, -90.0, 0.0, 1.0, 0.0, tetra_color2,
		   polymer_shininess, glutSolidTetrahedron);
	tetra2_xyzw = glGenLists(1);
	if (tetra2_xyzw == 0) goto err_handler;
	TETRA_LIST(tetra2_xyzw,  90.0, 0.0, 1.0, 0.0, tetra_color2,
		   polymer_shininess, glutSolidTetrahedron);
	
	/*****/
	/* WIRE TETRAHEDRON */
	tetraw_xzyw = glGenLists(1);
	if (tetraw_xzyw == 0) goto err_handler;
	TETRA_LIST(tetraw_xzyw,   0.0, 0.0, 1.0, 0.0, tetra_color1w,
		   polymer_shininessw, glutWireTetrahedron);
	tetraw_xwyz = glGenLists(1);
	if (tetraw_xwyz == 0) goto err_handler;
	TETRA_LIST(tetraw_xwyz, 180.0, 0.0, 1.0, 0.0, tetra_color1w,
		   polymer_shininessw, glutWireTetrahedron);
	tetraw_xzwy = glGenLists(1);
	if (tetraw_xzwy == 0) goto err_handler;
	TETRA_LIST(tetraw_xzwy, -90.0, 1.0, 0.0, 0.0, tetra_color1w,
		   polymer_shininessw, glutWireTetrahedron);
	tetraw_xywz = glGenLists(1);
	if (tetraw_xywz == 0) goto err_handler;
	TETRA_LIST(tetraw_xywz,  90.0, 1.0, 0.0, 0.0, tetra_color1w,
		   polymer_shininessw, glutWireTetrahedron);
	tetraw_xwzy = glGenLists(1);
	if (tetraw_xwzy == 0) goto err_handler;
	TETRA_LIST(tetraw_xwzy, -90.0, 0.0, 1.0, 0.0, tetra_color1w,
		   polymer_shininessw, glutWireTetrahedron);
	tetraw_xyzw = glGenLists(1);
	if (tetraw_xyzw == 0) goto err_handler;
	TETRA_LIST(tetraw_xyzw,  90.0, 0.0, 1.0, 0.0, tetra_color1w,
		   polymer_shininessw, glutWireTetrahedron);
	
	tetra2w_xzyw = glGenLists(1);
	if (tetra2w_xzyw == 0) goto err_handler;
	TETRA_LIST(tetra2w_xzyw,   0.0, 0.0, 1.0, 0.0, tetra_color2w,
		   polymer_shininessw, glutWireTetrahedron);
	tetra2w_xwyz = glGenLists(1);
	if (tetra2w_xwyz == 0) goto err_handler;
	TETRA_LIST(tetra2w_xwyz, 180.0, 0.0, 1.0, 0.0, tetra_color2w,
		   polymer_shininessw, glutWireTetrahedron);
	tetra2w_xzwy = glGenLists(1);
	if (tetra2w_xzwy == 0) goto err_handler;
	TETRA_LIST(tetra2w_xzwy, -90.0, 1.0, 0.0, 0.0, tetra_color2w,
		   polymer_shininessw, glutWireTetrahedron);
	tetra2w_xywz = glGenLists(1);
	if (tetra2w_xywz == 0) goto err_handler;
	TETRA_LIST(tetra2w_xywz,  90.0, 1.0, 0.0, 0.0, tetra_color2w,
		   polymer_shininessw, glutWireTetrahedron);
	tetra2w_xwzy = glGenLists(1);
	if (tetra2w_xwzy == 0) goto err_handler;
	TETRA_LIST(tetra2w_xwzy, -90.0, 0.0, 1.0, 0.0, tetra_color2w,
		   polymer_shininessw, glutWireTetrahedron);
	tetra2w_xyzw = glGenLists(1);
	if (tetra2w_xyzw == 0) goto err_handler;
	TETRA_LIST(tetra2w_xyzw,  90.0, 0.0, 1.0, 0.0, tetra_color2w,
		   polymer_shininessw, glutWireTetrahedron);
	
	/* tetrahedron table for rendering */
	set_tetra_table();
	return;
	
err_handler:
	fprintf(STDERR, "create_display_list() : ERROR index unavailable\n");
	exit(1);
}
